#include <iostream>
#include "wtmf.hpp"
#include "wtmf-learn.hpp"
#include "wtmf-corpus.hpp"
#include "wtmf-model.hpp"
#include <itpp/base/random.h>
#include <time.h>
#include <string.h>

void train(char* data_file, char* model_file, int dim, double lambda, double wm, int iteration);
void test(char* data_file, char* model_file, char* output_file);


int main(int argc, char *argv[]) {
  time_t start_t, end_t;
  start_t = time(NULL);
  
  char* data_file = 0;
  char* model_file = 0;
  char* output_file = 0;
  if (argc != 8 && argc != 5) {
    cout << "To train a model: ./wtmf 1 data_file model_file dim lambda wm iteration\nFor test data: ./wtmf 0 data_file model_file output_file\n";
    exit(1);
  }

  //RNG_reset(1001);

  // 1. train a wtmf model
  if (strcmp(argv[1],"1") == 0) {
    data_file = argv[2];
    cout << "[run_wtmf.cpp main()]: data_file = " << data_file << endl;
    model_file = argv[3];
    cout << "[run_wtmf.cpp main()]: model_file = " << model_file << endl;
    int dim = atoi(argv[4]);
    cout << "[run_wtmf.cpp main()]: dim = " << dim << endl;
    double lambda = atof(argv[5]);
    cout << "[run_wtmf.cpp main()]: lambda = " << lambda << endl;
    double wm = atof(argv[6]);
    cout << "[run_wtmf.cpp main()]: wm = " << wm << endl;
    int iteration = atof(argv[7]);
    cout << "[run_wtmf.cpp main()]: iteration = " << iteration << endl;
    train(data_file, model_file, dim, lambda, wm, iteration);
  }
  // 2. use an existing wtmf model file for inference
  else {
    data_file = argv[2];
    cout << "[run_wtmf.cpp main()]: data_file = " << data_file << endl;
    model_file = argv[3];
    cout << "[run_wtmf.cpp main()]: model_file = " << model_file << endl;
    output_file = argv[4];
    cout << "[run_wtmf.cpp main()]: output_file = " << output_file << endl;
    test(data_file, model_file, output_file);
  }

  end_t = time(NULL);
  int used_time = difftime(end_t, start_t);
  cout << "[run_wtmf.cpp main()]: used time = " << used_time << " seconds" << endl;
}




void train(char* data_file, char* model_file, int dim, double lambda, double wm, int iteration) {
  wtmf_corpus corpus;
  wtmf_model model;

  Sparse_Mat<double> data_mat;
  read_smat_data(data_file, data_mat);
  initialize_wtmf_corpus(corpus, data_mat);
  initialize_wtmf_model(model, corpus.sent_data.cols(), corpus.sent_data.rows(), dim, lambda, wm);
  
  wtmf_estimate(corpus, model, iteration);
  write_wtmf_model(model_file, model);
  delete_wtmf_corpus(corpus);
}




void test(char* data_file, char* model_file, char* output_file) {
  wtmf_corpus corpus;
  wtmf_model model;

  Sparse_Mat<double> data_mat;
  read_smat_data(data_file, data_mat);
  initialize_wtmf_corpus(corpus, data_mat);
  read_wtmf_model(model_file, model);
  
  Mat<double> q;
  wtmf_inference(corpus, model, q);
  write_mat_data(output_file, q);
  delete_wtmf_corpus(corpus);
}
